//  
//  Copyright (C) 2010 Claudio Melis
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
// 

using System;
using System.Collections.Generic;
using System.Diagnostics;

using NDesk.DBus;
using org.freedesktop.DBus;

using Mono.Unix;

using Docky.Items;
using Docky.Menus;
using Docky.Services;

namespace SessionManager
{
	public class SessionManagerType
	{
       public string icon, hover_text;
		
	   public SessionManagerType(string icon, string hover_text) 
	   {
	      this.icon = icon;
	      this.hover_text = hover_text;
	   }
	}
	
	public class SessionManagerItem : IconDockItem
	{				
		const int num_items = 6;
		int actual_index = 0;	
		List<SessionManagerType> SessionItems;
		
		const string DeviceKitPowerName = "org.freedesktop.DeviceKit.Power";
		const string DeviceKitPowerPath = "/org/freedesktop/DeviceKit/Power";
		
		const string ConsoleKitName = "org.freedesktop.ConsoleKit";
		const string ConsoleKitPath = "/org/freedesktop/ConsoleKit/Manager";
		
		const string UPowerName = "org.freedesktop.UPower";
		const string UPowerPath = "/org/freedesktop/UPower";
		
		static object power_manager;
		static object system_manager;
		
		[Interface (DeviceKitPowerName)]
		interface IDeviceKitPower
		{
			void Hibernate ();
			void Suspend ();
		}
		
		[Interface (UPowerName)]
		interface IUPower
		{
			void Hibernate ();
			void Suspend ();
		}

		[Interface ("org.freedesktop.ConsoleKit.Manager")]
		interface IConsoleKit
		{
			void Stop ();
			void Restart ();
		}
		
		void initializeBus ()    
		{
			try {
				// Init Power Manager Bus
				if (Bus.System.NameHasOwner (UPowerName)) {
					power_manager = Bus.System.GetObject<IUPower> (UPowerName, new ObjectPath (UPowerPath));
				} else if (Bus.System.NameHasOwner (DeviceKitPowerName)) {
					power_manager = Bus.System.GetObject<IDeviceKitPower> (DeviceKitPowerName, new ObjectPath (DeviceKitPowerPath));
				}
			} catch (Exception e) {
				Log<SessionManagerItem>.Error ("Could not initialize Power Manager Bus: {0}", e.Message);
				Log<SessionManagerItem>.Debug (e.StackTrace);
			}
			try {
				// Init System Manager Bus
				system_manager = Bus.System.GetObject<IConsoleKit> (ConsoleKitName, new ObjectPath (ConsoleKitPath));
			} catch (Exception e) {
				Log<SessionManagerItem>.Error ("Could not initialize System Manager Bus: {0}", e.Message);
				Log<SessionManagerItem>.Debug (e.StackTrace);			
			}
		}

		public SessionManagerItem ()
		{			
			initializeBus();
			SessionItems = new List<SessionManagerType> ();
			
            SessionItems.Add (new SessionManagerType("gnome-lockscreen", Catalog.GetString ("Lock Screen")));      
			SessionItems.Add (new SessionManagerType("gnome-session-logout", Catalog.GetString ("Log Out..."))); 
			SessionItems.Add (new SessionManagerType("gnome-session-sleep", Catalog.GetString ("Suspend"))); 
			SessionItems.Add (new SessionManagerType("gnome-session-hibernate", Catalog.GetString ("Hibernate"))); 
			SessionItems.Add (new SessionManagerType("gnome-session-reboot", Catalog.GetString ("Restart"))); 
			SessionItems.Add (new SessionManagerType("gnome-session-halt", Catalog.GetString ("Shut Down"))); 
			
			HoverText = SessionItems[actual_index].hover_text;
			Icon = SessionItems[actual_index].icon;
		}
		
		public override string UniqueID ()
		{
			return "SessionManager";
		}
		
		protected override ClickAnimation OnClicked (uint button, Gdk.ModifierType mod, double xPercent, double yPercent)
		{
			if (button == 1) {
				setAction(actual_index);
				return ClickAnimation.Bounce;
			}
			return ClickAnimation.None;
		}
		
		public static void setAction(int index)
		{
				Log<SessionManagerItem>.Debug("dentro setAction");
				Log<SessionManagerItem>.Debug("{0}", index);
				switch (index) 
				{
				case 0:
					SessionManagerCommand (0); // ScreenSaver
					break;
				case 1:
					SessionManagerCommand (1); // EndSession
					break;
				case 2:
					Suspend ();
					break;
				case 3:
					Hibernate ();
					break;
				case 4:
					Restart ();
					break;
				case 5:
				    Shutdown ();
					break;
				}
		}
		
		protected override MenuList OnGetMenuItems ()
		{
			MenuList list = new MenuList ();
			for (int i = 0; i < num_items; i++)
			{
				int j = i;
			Log<SessionManagerItem>.Debug("{0}", i);
			list[MenuListContainer.Actions].Add (
				new MenuItem (Catalog.GetString (SessionItems[i].hover_text), SessionItems[i].icon, (o, a) => setAction(j)));
			}/*
			list[MenuListContainer.Actions].Add (
				new MenuItem (Catalog.GetString (SessionItems[1].hover_text), SessionItems[1].icon, (o, a) => SessionManagerCommand(1)));
			list[MenuListContainer.Actions].Add (
				new MenuItem (Catalog.GetString (SessionItems[2].hover_text), SessionItems[2].icon, (o, a) => Suspend()));
			list[MenuListContainer.Actions].Add (
				new MenuItem (Catalog.GetString (SessionItems[3].hover_text), SessionItems[3].icon, (o, a) => Hibernate()));
			list[MenuListContainer.Actions].Add (
				new MenuItem (Catalog.GetString (SessionItems[4].hover_text), SessionItems[4].icon, (o, a) => Restart()));
			list[MenuListContainer.Actions].Add (
				new MenuItem (Catalog.GetString (SessionItems[5].hover_text), SessionItems[5].icon, (o, a) => Shutdown()));
			*/
			return list;
		}
		
		public static void SessionManagerCommand (int index) // work
		{
			string command = "";
			string args = "";
			switch (index)
			{
			case 0:
				command = "gnome-screensaver-command";
				args = "--lock";
				break;
			case 1:
				command = "gnome-session-save";
				args = "--logout-dialog";
				break;
			default:
				break;
			}
			try {
				System.Diagnostics.Process.Start (command, args);
			} catch (Exception e) {
				Log<SessionManagerItem>.Error ("Could not lauch "+command+" Command.");
				Log<SessionManagerItem>.Debug (e.StackTrace);
			}
				
		}
					
		public static void Suspend () // work
		{	
			try {
				SessionManagerCommand (0);
				if (power_manager is IDeviceKitPower) {
					(power_manager as IDeviceKitPower).Suspend ();
				} else if (power_manager is IUPower) {
					(power_manager as IUPower).Suspend ();
				}
			} catch (Exception e) {
				Log<SessionManagerItem>.Error ("Could not suspend: {0}", e.Message);
				Log<SessionManagerItem>.Debug (e.StackTrace);
			}
		}
		
		public static void Hibernate () // work
		{
			try {
				SessionManagerCommand (0);
				if (power_manager is IDeviceKitPower) {
					(power_manager as IDeviceKitPower).Hibernate ();
				} else if (power_manager is IUPower) {
					(power_manager as IUPower).Hibernate ();
				}
			} catch (Exception e) {
				Log<SessionManagerItem>.Error ("Could not hibernate: {0}", e.Message);
				Log<SessionManagerItem>.Debug (e.StackTrace);
			}
		}
		
		public static void Shutdown () // work
		{
			try {
				(system_manager as IConsoleKit).Stop ();
			} catch (Exception e) {
				Log<SessionManagerItem>.Error ("Could not shutdown: {0}", e.Message);
				Log<SessionManagerItem>.Debug (e.StackTrace);
			}
		}
		
		public static void Restart () // to test
		{
			try {
				(system_manager as IConsoleKit).Restart ();
			} catch (Exception e) {
				Log<SessionManagerItem>.Error ("Could not restart: {0}", e.Message);
				Log<SessionManagerItem>.Debug (e.StackTrace);
			}
		}
		
		protected override void OnScrolled (Gdk.ScrollDirection direction, Gdk.ModifierType mod)
		{
			if (direction == Gdk.ScrollDirection.Up) {
				if ( actual_index == 0 ) {
					actual_index = num_items - 1;
				} else {
					actual_index--;
				}
			} else {	
				if ( actual_index == num_items - 1 ) {
					actual_index = 0;
				} else {
					actual_index++;
				}
			}
			HoverText = SessionItems[actual_index].hover_text;
			Icon = SessionItems[actual_index].icon;
			QueueRedraw ();
		}
				
		#region IDisposable implementation
		public override void Dispose ()
		{
			base.Dispose ();
		}

		#endregion
	}
}